# 
#  quiz.awk  Ver 1.4.1
#
#  <C> Copyright 1995 Symbiotic Solutions, InterSpect
#  Authors:  Emanuel Noik, Andrew Csinger
#
#  This code is actually gawk, not awk!
#
#  This script parses a ".q" quiz format input file and generates several
#  output files that together allow users to play simple interactive quizes
#  using World-Wide Web browsers.  In particular:
#
#  1. a questions form - html
#  2. an answer list - html
#  3. a CGI database file for the quiz engine (below) - custom format
#  4. answer explanations - html
#
#  Note that there is a companion quiz.c program that is used to evaluate
#  users' answers, compute totals, build html response page, etc.
#

BEGIN {
   FS = "|";		# field separator
   qn = -1;		# count queries
   sn = -1;		# count scoring categories
}

# db root name background
#
$1 == "db" {
   root = $2;		# Note root must be same as in
			# Makefile used to make the quiz executable
   qname = $3;
   fd = sprintf( "%s/%s", root, qname );
   bgd = $4;
}

# q question_url question_file question_background
#
$1 == "q" {
   url_q = $2;
   fq = $3;
   bgq = $4;
}

# a answer_url answer_file answer_background
#
$1 == "a" {
   url_a = $2;
   fa = $3;
   bga = $4;
}

# e explanation_url explanation_file explanation_background
#
$1 == "e" {
   url_e = $2;
   fe = $3;
   bge = $4;
}

# t form_title form_header
#
$1 == "t" {
   title = $2;
   titleh = $3;
   key = "t";
}

# s section_label
#
$1 == "s" {
   section[qn] = $2;
   key = sprintf( "s,%d", qn );
}

# qu question_text explanation_text
#
$1 == "qu" {
   qn++;
   on = 0;	# counts options
   question[qn] = $2;
   explanation[qn] = $3;
   key = sprintf( "qu,%d", qn );
}

# uq
#
$1 == "uq" {
   optn[qn] = on;
}

# o label checked? correct?
#
$1 == "o" {
   option_label[qn,on] = $2;
   option_checked[qn,on] = $3;
   option_correct[qn,on] = $4;
   key = sprintf( "o,%d,%d", qn, on );
   on++;
}

# scoring label
#
$1 == "scoring" {
   scoring = $2;
   key = "scoring";
}

# score range_low range_hi|999 label
#
$1 == "score" {
   sn++;
   score_lo[sn] = $2;
   score_hi[sn] = $3;
   score_label[sn] = $4;
   key = sprintf( "score,%d", sn );
}

# i q a e content
#
$1 == "i" {
   if ( $2 == "1" ) { k = sprintf( "q,%s", key ); inlined[k] = $5; }
   if ( $3 == "1" ) { k = sprintf( "a,%s", key ); inlined[k] = $5; }
   if ( $4 == "1" ) { k = sprintf( "e,%s", key ); inlined[k] = $5; }
}

#-------------------------------------------------------------------------------

function PrintNonEmpty( s, pre, post, r, f ) {
   if( s != "" ) print pre s post >> f;
   else if( r != "" ) print r >> f;
}

function PrintHeader( t1, t2, t3, h, bg, ftype, f,     key ) {
   print "<html>"						>  f;
   print "<head><title>" t1 t2 t3 "</title></head>"		>> f;
   PrintNonEmpty( bg, "<body background=\"", "\">", "<body>", f );
   key = sprintf( "%s,t", ftype );
   PrintNonEmpty( inlined[key], "", "", "", f );
   print "<h2>" h t2 t3 "</h2>"					>> f;
}

function PrintFooter( f ) {
   print "</body>"	>> f
   print "</html>"	>> f
}

function PrintSection( i, ftype, f,      key ) {
   key = sprintf( "%s,s,%d", ftype, i - 1 );
   PrintNonEmpty( inlined[key], "", "", "", f );
   PrintNonEmpty( section[i-1], "<p><hr><h3>", "</h3>", "", f );
}

function PrintQuestion( i,     key ) {
   key = sprintf( "q,qu,%d", i );
   PrintNonEmpty( inlined[key], "", "", "", fq );
   print "<p><li><i>" question[i] "</i><br>" >> fq
}

function PrintQuestionOption( i, j,     key, ch ) {
   key = sprintf( "q,o,%d,%d", i, j );
   PrintNonEmpty( inlined[key], "", "", "", fq );
   ch = (option_checked[i,j] != 0) ? "checked" : "";
   print "<input type=\"radio\" name=\"" i "\" value=\"" j "\"" ch ">" \
   " " option_label[i,j] "<br>" >> fq
}

function PrintAnswer( i,     key ) {
   key = sprintf( "a,qu,%d", i );
   PrintNonEmpty( inlined[key], "", "", "", fa );
   print "<p><li><a href=\"" url_e "#" i "\"><i>" question[i] \
   "</i></a><br>" >> fa
}

function PrintAnswerOption( i, j,     key ) {
   key = sprintf( "a,o,%d,%d", i, j );
   PrintNonEmpty( inlined[key], "", "", "", fq );
   if ( option_correct[i,j] != 0 ) {
      print "<font size=\"+1\"><strong>" option_label[i,j] \
      "</strong></font><br>" >> fa;
   }
   else {
      print option_label[i,j] "<br>" >> fa;
   }
}

function PrintExplanation( i,     key ) {
   key = sprintf( "e,qu,%d", i );
   PrintNonEmpty( inlined[key], "", "", "", fa );
   print "<p><li><a name=\"" i "\"><i>" question[i] "</i></a><br>" >> fe;
}

function PrintExplanationOption( i, j,     key ) {
   if ( option_correct[i,j] != 0 ) {
      key = sprintf( "e,o,%d,%d", i, j );
      PrintNonEmpty( inlined[key], "", "", "", fq );
      print "<b>" option_label[i,j] "</b><br>" >> fe;
   }
}

function PrintScoreHeader(     key ) {
   print "<p><hr>" >> fa;
   key = "a,scoring";
   PrintNonEmpty( inlined[key], "", "", "", fa );
   print "<h3>" scoring "</h3>" >> fa;
}

function PrintScoreCategory( i,     key ) {
   key = sprintf( "a,score,%d", i );
   PrintNonEmpty( inlined[key], "", "", "", fa );
   if ( score_hi[i] < 999 )
      print "<dt>" score_lo[i] " - " score_hi[i] " <dd><i>" score_label[i] \
         "</i>" >> fa;
   else
      print "<dt>" score_lo[i] " - <dd><i>" score_label[i] "</i>" >> fa;
}

#-------------------------------------------------------------------------------

END {

  #  Questions file.
  #
  PrintHeader( title, "", "", titleh, bgq, "q", fq );

  print "<form method=\"post\" action=\"/cgi-bin/quiz?" qname "\">"	>> fq
  print "<ol>"								>> fq

  for ( i = 0; i <= qn; i++ ) {
     PrintSection( i, "q", fq );
     PrintQuestion( i );
     for ( j = 0; j < optn[i]; j++ ) {
        PrintQuestionOption( i, j );
     }
  }

  print "</ol>"								>> fq
  print "<p><hr><input type=\"submit\" value=\"Done\">"			>> fq
  print "<input type=\"reset\" value=\"Clear\">"			>> fq
  print "</form>"							>> fq
  PrintFooter( fq );

  #  Answer file.
  #
  PrintHeader( title, " - ", "Answers", titleh, bga, "a", fa );

  print "<ol>"								>> fa
  for ( i = 0; i <= qn; i++ ) {
     PrintSection( i, "a", fa );
     PrintAnswer( i );
     for ( j = 0; j < optn[i]; j++ ) {
        PrintAnswerOption( i, j );
     }
  }
  print "</ol>"								>> fa

  PrintScoreHeader();
  print "<dl>"								>> fa
  for ( i = 0; i <= sn; i++ ) {
     PrintScoreCategory( i );
  }
  print "</dl>"								>> fa
  PrintFooter( fa );

  #  Explanations file.
  #
  PrintHeader( title, " - ", "Explanations", titleh, bge, "e", fe );

  print "<ol>"								>> fe
  for ( i = 0; i <= qn; i++ ) {
     PrintSection( i, "e", fe );
     PrintExplanation( i );
     for ( j = 0; j < optn[i]; j++ ) {
        PrintExplanationOption( i, j );
     }
     PrintNonEmpty( explanation[i], "<i>Reason:</i> ", "", "", fe );
  }
  print "</ol>"								>> fe
  PrintFooter( fe );

  #  Database file.
  #
  print "q " url_q							>  fd
  print "a " url_a							>> fd
  print "e " url_e							>> fd
  print "b " bgd							>> fd

  for ( i = 0; i <= qn; i++ ) {
     for ( j = 0; j < optn[i]; j++ ) {
        print "o " i " " j " " option_correct[i,j] " " option_misc[i,j]	>> fd
     }
  }

  for ( i = 0; i <= sn; i++ ) {
     print "s " score_lo[i] " " score_hi[i] " {" score_label[i] "}"	>> fd
  }
}

#
#  end of:  quiz.awk
